# -*- coding: utf8 -*-
# Dioptas - GUI program for fast processing of 2D X-ray data
# Copyright (C) 2017  Clemens Prescher (clemens.prescher@gmail.com)
# Institute for Geology and Mineralogy, University of Cologne
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import unittest
import gc
import os
import numpy as np
from math import sqrt, atan2, cos, sin
from qtpy import QtCore

from ...model.MaskModel import MaskModel

from ..utility import delete_if_exists

unittest_path = os.path.dirname(__file__)
data_path = os.path.join(unittest_path, '../data')


class MaskModelTest(unittest.TestCase):
    def setUp(self):
        self.mask_model = MaskModel()
        self.img = np.zeros((10, 10))
        self.mask_model.set_dimension(self.img.shape)

    def tearDown(self):
        delete_if_exists(os.path.join(data_path, "test_save.mask"))
        del self.mask_model
        del self.img
        gc.collect()

    def test_growing_masks(self):
        self.mask_model._mask_data[4, 4] = 1
        self.mask_model._mask_data[0, 0] = 1
        self.mask_model._mask_data[0, 9] = 1
        self.mask_model._mask_data[9, 9] = 1
        self.mask_model._mask_data[9, 0] = 1

        self.mask_model.grow()

        # tests corners
        self.assertEqual(self.mask_model._mask_data[0, 1], 1)
        self.assertEqual(self.mask_model._mask_data[1, 1], 1)
        self.assertEqual(self.mask_model._mask_data[1, 0], 1)

        self.assertEqual(self.mask_model._mask_data[0, 8], 1)
        self.assertEqual(self.mask_model._mask_data[1, 8], 1)
        self.assertEqual(self.mask_model._mask_data[1, 9], 1)

        self.assertEqual(self.mask_model._mask_data[8, 0], 1)
        self.assertEqual(self.mask_model._mask_data[8, 1], 1)
        self.assertEqual(self.mask_model._mask_data[9, 1], 1)

        self.assertEqual(self.mask_model._mask_data[8, 8], 1)
        self.assertEqual(self.mask_model._mask_data[8, 9], 1)
        self.assertEqual(self.mask_model._mask_data[9, 8], 1)

        # tests center
        self.assertEqual(self.mask_model._mask_data[3, 3], 1)
        self.assertEqual(self.mask_model._mask_data[4, 3], 1)
        self.assertEqual(self.mask_model._mask_data[5, 3], 1)

        self.assertEqual(self.mask_model._mask_data[3, 5], 1)
        self.assertEqual(self.mask_model._mask_data[4, 5], 1)
        self.assertEqual(self.mask_model._mask_data[5, 5], 1)

        self.assertEqual(self.mask_model._mask_data[3, 4], 1)
        self.assertEqual(self.mask_model._mask_data[5, 4], 1)

    def test_shrink_mask(self):
        self.mask_model._mask_data[4, 4] = 1
        self.mask_model._mask_data[0, 0] = 1
        self.mask_model._mask_data[0, 9] = 1
        self.mask_model._mask_data[9, 9] = 1
        self.mask_model._mask_data[9, 0] = 1

        self.before_mask = np.copy(self.mask_model._mask_data)
        self.mask_model.grow()
        self.mask_model.shrink()

        self.assertTrue(np.array_equal(self.before_mask, self.mask_model._mask_data))

        self.mask_model.clear_mask()

        self.mask_model._mask_data[4, 4] = 1
        self.mask_model._mask_data[5, 4] = 1
        self.mask_model._mask_data[5, 5] = 1
        self.mask_model._mask_data[4, 5] = 1
        self.mask_model.shrink()

        self.assertEqual(np.sum(self.mask_model._mask_data), 0)

    def test_saving_and_loading(self):
        self.mask_model.mask_ellipse(1024, 1024, 100, 100)
        self.mask_model.set_dimension((2048, 2048))

        mask_array = np.copy(self.mask_model.get_img())

        filename = os.path.join(data_path, 'dummy.mask')

        self.mask_model.save_mask(filename)
        self.mask_model.load_mask(filename)

        self.assertTrue(np.array_equal(mask_array, self.mask_model.get_img()))
        os.remove(filename)

    def test_use_roi(self):
        self.mask_model.roi = [0, 2, 0, 2]

        self.assertTrue(np.array_equal(self.mask_model.get_mask()[0:3, 0:3],
                                       np.array([[0, 0, 1],
                                                 [0, 0, 1],
                                                 [1, 1, 1]]))
                        )

    def test_use_roi_with_supersampling(self):
        self.mask_model.roi = [0, 2, 0, 2]
        self.mask_model.set_supersampling(2)
        self.assertTrue(np.array_equal(self.mask_model.get_mask()[0:6, 0:6],
                                       np.array([[0, 0, 0, 0, 1, 1],
                                                 [0, 0, 0, 0, 1, 1],
                                                 [0, 0, 0, 0, 1, 1],
                                                 [0, 0, 0, 0, 1, 1],
                                                 [1, 1, 1, 1, 1, 1],
                                                 [1, 1, 1, 1, 1, 1]]))
                        )

    def test_save_mask(self):
        self.mask_model.mask_below_threshold(self.img, 1)
        self.mask_model.save_mask(os.path.join(data_path, "test_save.mask"))

        self.assertTrue(os.path.exists(os.path.join(data_path, "test_save.mask")))

    def test_find_center_of_circle_from_three_points(self):
        x0 = 2.0
        y0 = 3.5
        r = 1.2
        phi1 = 0.1
        phi2 = 1.3
        phi3 = 6.0
        p1 = QtCore.QPointF(x0 + r * cos(phi1), y0 + r * sin(phi1))
        p2 = QtCore.QPointF(x0 + r * cos(phi2), y0 + r * sin(phi2))
        p3 = QtCore.QPointF(x0 + r * cos(phi3), y0 + r * sin(phi3))
        # p1 = (x0 + r * cos(phi1), y0 + r * sin(phi1))
        # p2 = (x0 + r * cos(phi2), y0 + r * sin(phi2))
        # p3 = (x0 + r * cos(phi3), y0 + r * sin(phi3))
        self.mask_model.find_center_of_circle_from_three_points(p1, p2, p3)
        self.assertAlmostEqual(x0, self.mask_model.center_for_arc.x(), 6)
        self.assertAlmostEqual(y0, self.mask_model.center_for_arc.y(), 6)

    def test_find_radius_of_circle_from_center_and_point(self):
        x0 = 2.0
        y0 = 3.5
        p0 = QtCore.QPointF(x0, y0)
        r = 1.2
        phi1 = 0.1
        p1 = QtCore.QPointF(x0 + r * cos(phi1), y0 + r * sin(phi1))
        rcalc = self.mask_model.find_radius_of_circle_from_center_and_point(p0, p1)
        self.assertEqual(r, rcalc)

    def test_find_n_points_on_arc_from_three_points(self):
        n = 50
        x0 = 2.0
        y0 = 3.5
        p0 = QtCore.QPointF(x0, y0)
        r = 1.2
        width = 0

        phi1 = 0.1
        phi2 = 1.3
        phi3 = -0.2
        p1 = QtCore.QPointF(x0 + r * cos(phi1), y0 + r * sin(phi1))
        p2 = QtCore.QPointF(x0 + r * cos(phi2), y0 + r * sin(phi2))
        p3 = QtCore.QPointF(x0 + r * cos(phi3), y0 + r * sin(phi3))

        n_angles = self.mask_model.find_n_angles_on_arc_from_three_points_around_p0(p0, p1, p2, p3, n)
        n_points = self.mask_model.calc_arc_points_from_angles(p0, r, width, n_angles)
        for p in n_points:
            rcalc = self.mask_model.find_radius_of_circle_from_center_and_point(p0, p)
            self.assertAlmostEqual(r, rcalc, 5)



# -*- coding: utf8 -*-
# Dioptas - GUI program for fast processing of 2D X-ray data
# Copyright (C) 2017  Clemens Prescher (clemens.prescher@gmail.com)
# Institute for Geology and Mineralogy, University of Cologne
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from ..utility import QtTest, click_button
import os
import gc
from mock import MagicMock

import numpy as np
from qtpy import QtWidgets, QtCore
from qtpy.QtTest import QTest

from ...controller.integration import OverlayController
from ...model.DioptasModel import DioptasModel
from ...widgets.integration import IntegrationWidget

unittest_path = os.path.dirname(__file__)
data_path = os.path.join(unittest_path, '../data')


class OverlayControllerTest(QtTest):

    def setUp(self):
        self.integration_widget = IntegrationWidget()
        self.overlay_widget = self.integration_widget.overlay_widget
        self.model = DioptasModel()
        self.model.working_directories['overlay'] = data_path
        self.overlay_controller = OverlayController(self.integration_widget, self.model)
        self.overlay_tw = self.integration_widget.overlay_tw

    def tearDown(self):
        del self.overlay_controller
        del self.overlay_tw
        del self.integration_widget
        del self.model
        gc.collect()

    def test_manual_deleting_overlays(self):
        self.load_overlays()

        self.assertEqual(self.overlay_tw.rowCount(), 6)
        self.assertEqual(len(self.model.overlay_model.overlays), 6)
        self.assertEqual(len(self.integration_widget.pattern_widget.overlays), 6)
        self.assertEqual(self.overlay_tw.currentRow(), 5)

        self.overlay_controller.delete_btn_click_callback()
        self.assertEqual(self.overlay_tw.rowCount(), 5)
        self.assertEqual(len(self.model.overlay_model.overlays), 5)
        self.assertEqual(len(self.integration_widget.pattern_widget.overlays), 5)
        self.assertEqual(self.overlay_tw.currentRow(), 4)

        self.overlay_widget.select_overlay(1)
        self.overlay_controller.delete_btn_click_callback()
        self.assertEqual(self.overlay_tw.rowCount(), 4)
        self.assertEqual(len(self.model.overlay_model.overlays), 4)
        self.assertEqual(len(self.integration_widget.pattern_widget.overlays), 4)
        self.assertEqual(self.overlay_tw.currentRow(), 1)

        self.overlay_widget.select_overlay(0)
        self.overlay_controller.delete_btn_click_callback()
        self.assertEqual(self.overlay_tw.rowCount(), 3)
        self.assertEqual(len(self.model.overlay_model.overlays), 3)
        self.assertEqual(len(self.integration_widget.pattern_widget.overlays), 3)
        self.assertEqual(self.overlay_tw.currentRow(), 0)

        self.overlay_controller.delete_btn_click_callback()
        self.overlay_controller.delete_btn_click_callback()
        self.overlay_controller.delete_btn_click_callback()
        self.assertEqual(self.overlay_tw.rowCount(), 0)
        self.assertEqual(len(self.model.overlay_model.overlays), 0)
        self.assertEqual(len(self.integration_widget.pattern_widget.overlays), 0)
        self.assertEqual(self.overlay_tw.currentRow(), -1)

        self.overlay_controller.delete_btn_click_callback()
        self.assertEqual(self.overlay_tw.rowCount(), 0)
        self.assertEqual(self.overlay_tw.currentRow(), -1)

    def test_automatic_deleting_overlays(self):
        self.load_overlays()
        self.load_overlays()
        QtWidgets.QApplication.processEvents()
        QtWidgets.QApplication.processEvents()
        self.assertEqual(self.overlay_tw.rowCount(), 12)
        self.assertEqual(len(self.model.overlay_model.overlays), 12)
        self.assertEqual(len(self.integration_widget.pattern_widget.overlays), 12)

        self.overlay_controller.clear_overlays_btn_click_callback()
        self.assertEqual(self.overlay_tw.rowCount(), 0)
        self.assertEqual(len(self.model.overlay_model.overlays), 0)
        self.assertEqual(len(self.integration_widget.pattern_widget.overlays), 0)
        self.assertEqual(self.overlay_tw.currentRow(), -1)

        multiplier = 1
        for dummy_index in range(multiplier):
            self.load_overlays()

        self.assertEqual(self.overlay_tw.rowCount(), multiplier * 6)
        self.overlay_controller.clear_overlays_btn_click_callback()
        self.assertEqual(self.overlay_tw.rowCount(), 0)
        self.assertEqual(len(self.model.overlay_model.overlays), 0)
        self.assertEqual(len(self.integration_widget.pattern_widget.overlays), 0)
        self.assertEqual(self.overlay_tw.currentRow(), -1)

    def test_change_scaling_in_view(self):
        self.load_overlays()
        self.overlay_widget.select_overlay(2)

        self.integration_widget.overlay_scale_sb.setValue(2.0)
        self.app.processEvents()
        self.assertEqual(self.model.overlay_model.get_overlay_scaling(2), 2)

        # tests if overlay is updated in pattern
        x, y = self.model.overlay_model.overlays[2].data
        x_spec, y_spec = self.integration_widget.pattern_widget.overlays[2].getData()

        self.assertAlmostEqual(np.sum(y - y_spec), 0)

    def test_change_offset_in_view(self):
        self.load_overlays()
        self.overlay_widget.select_overlay(3)

        self.integration_widget.overlay_offset_sb.setValue(100)
        self.assertEqual(self.model.overlay_model.get_overlay_offset(3), 100)

        x, y = self.model.overlay_model.overlays[3].data
        x_spec, y_spec = self.integration_widget.pattern_widget.overlays[3].getData()

        self.assertAlmostEqual(np.sum(y - y_spec), 0)

    def test_scaling_auto_step_change(self):
        self.load_overlays()
        self.overlay_widget.scale_step_msb.setValue(0.5)
        self.overlay_widget.scale_step_msb.stepUp()

        new_scale_step = self.overlay_widget.scale_step_msb.value()
        self.assertAlmostEqual(new_scale_step, 1.0, places=5)

        self.overlay_widget.scale_step_msb.stepDown()
        self.overlay_widget.scale_step_msb.stepDown()
        new_scale_step = self.overlay_widget.scale_step_msb.value()
        self.assertAlmostEqual(new_scale_step, 0.2, places=5)

    def test_offset_auto_step_change(self):
        self.load_overlays()
        self.overlay_widget.offset_step_msb.setValue(10.0)
        self.overlay_widget.offset_step_msb.stepUp()

        new_offset_step = self.overlay_widget.offset_step_msb.value()
        self.assertAlmostEqual(new_offset_step, 20.0, places=5)

        self.overlay_widget.offset_step_msb.stepDown()
        self.overlay_widget.offset_step_msb.stepDown()
        new_offset_step = self.overlay_widget.offset_step_msb.value()
        self.assertAlmostEqual(new_offset_step, 5.0, places=5)

    def test_setting_overlay_as_bkg(self):
        self.load_overlays()
        self.model.pattern_model.load_pattern(os.path.join(data_path, 'pattern_001.xy'))
        self.overlay_widget.select_overlay(0)
        QTest.mouseClick(self.integration_widget.overlay_set_as_bkg_btn, QtCore.Qt.LeftButton)

        self.assertTrue(self.integration_widget.overlay_set_as_bkg_btn.isChecked())

        self.assertEqual(self.model.pattern_model.background_pattern, self.model.overlay_model.overlays[0])
        x, y = self.model.pattern.data
        self.assertEqual(np.sum(y), 0)

    def test_setting_overlay_as_bkg_and_changing_scale(self):
        self.load_overlays()
        self.model.pattern_model.load_pattern(os.path.join(data_path, 'pattern_001.xy'))
        self.overlay_widget.select_overlay(0)
        QTest.mouseClick(self.integration_widget.overlay_set_as_bkg_btn, QtCore.Qt.LeftButton)

        self.integration_widget.overlay_scale_sb.setValue(2)
        _, y = self.model.pattern.data
        _, y_original = self.model.pattern.data
        self.assertEqual(np.sum(y - y_original), 0)

    def test_setting_overlay_as_bkg_and_changing_offset(self):
        self.load_overlays()
        self.model.pattern_model.load_pattern(os.path.join(data_path, 'pattern_001.xy'))
        self.overlay_widget.select_overlay(0)
        QTest.mouseClick(self.integration_widget.overlay_set_as_bkg_btn, QtCore.Qt.LeftButton)

        self.integration_widget.overlay_offset_sb.setValue(100)
        _, y = self.model.pattern.data
        self.assertEqual(np.sum(y), -100 * y.size)

    def test_setting_overlay_as_bkg_and_then_change_to_new_overlay_as_bkg(self):
        self.load_overlays()
        self.model.pattern_model.load_pattern(os.path.join(data_path, 'pattern_001.xy'))
        self.overlay_widget.select_overlay(0)
        QTest.mouseClick(self.integration_widget.overlay_set_as_bkg_btn, QtCore.Qt.LeftButton)

        _, y = self.model.pattern.data
        self.assertEqual(np.sum(y), 0)

        self.overlay_widget.select_overlay(1)
        self.integration_widget.overlay_scale_sb.setValue(2)
        QTest.mouseClick(self.integration_widget.overlay_set_as_bkg_btn, QtCore.Qt.LeftButton)

        _, y = self.model.pattern.data
        self.assertNotEqual(np.sum(y), 0)

    def test_setting_pattern_as_bkg(self):
        self.model.pattern_model.load_pattern(os.path.join(data_path, 'pattern_001.xy'))
        QTest.mouseClick(self.integration_widget.qa_set_as_background_btn, QtCore.Qt.LeftButton)
        QtWidgets.QApplication.processEvents()

        self.assertTrue(self.integration_widget.overlay_set_as_bkg_btn.isChecked())

        _, y = self.model.pattern.data
        self.assertEqual(np.sum(y), 0)

    def test_having_overlay_as_bkg_and_deleting_it(self):
        self.model.pattern_model.load_pattern(os.path.join(data_path, 'pattern_001.xy'))
        QTest.mouseClick(self.integration_widget.qa_set_as_background_btn, QtCore.Qt.LeftButton)
        self.assertEqual(len(self.model.overlay_model.overlays), 1)
        self.assertIsNotNone(self.model.pattern_model.background_pattern)

        QTest.mouseClick(self.integration_widget.overlay_del_btn, QtCore.Qt.LeftButton)

        self.assertFalse(self.integration_widget.overlay_set_as_bkg_btn.isChecked())
        self.assertEqual(self.integration_widget.overlay_tw.rowCount(), 0)

        _, y = self.model.pattern.data
        self.assertNotEqual(np.sum(y), 0)

    def test_overlay_waterfall(self):
        self.load_overlays()
        self.overlay_widget.waterfall_separation_msb.setValue(10)
        QTest.mouseClick(self.overlay_widget.waterfall_btn, QtCore.Qt.LeftButton)

        self.assertEqual(self.model.overlay_model.overlays[5].offset, -10)
        self.assertEqual(self.model.overlay_model.overlays[4].offset, -20)

        QTest.mouseClick(self.integration_widget.reset_waterfall_btn, QtCore.Qt.LeftButton)

        self.assertEqual(self.model.overlay_model.overlays[5].offset, 0)
        self.assertEqual(self.model.overlay_model.overlays[5].offset, 0)

    def load_overlays(self):
        self.load_overlay('pattern_001.xy')
        self.load_overlay('pattern_001.xy')
        self.load_overlay('pattern_001.xy')
        self.load_overlay('pattern_001.xy')
        self.load_overlay('pattern_001.xy')
        self.load_overlay('pattern_001.xy')

    def load_overlay(self, filename):
        QtWidgets.QFileDialog.getOpenFileNames = MagicMock(return_value=[os.path.join(data_path, filename)])
        click_button(self.integration_widget.overlay_add_btn)

    def test_move_single_overlay_one_step_up(self):
        # setting up the test
        self.load_overlays()
        self.overlay_widget.select_overlay(3)
        new_name = 'special'
        self.assertEqual(self.integration_widget.overlay_tw.item(3, 2).text(), 'pattern_001')
        self.overlay_controller.rename_overlay(3, new_name)
        self.integration_widget.overlay_tw.item(3, 2).setText(new_name)
        self.assertEqual(self.model.overlay_model.overlays[3].name, new_name)
        self.assertEqual(self.integration_widget.overlay_tw.item(3, 2).text(), new_name)
        self.assertEqual(self.model.overlay_model.overlays[2].name, 'pattern_001')

        # moving the overlay
        self.integration_widget.overlay_move_up_btn.click()

        # test for wanted result
        self.assertEqual(self.model.overlay_model.overlays[2].name, new_name)
        self.assertEqual(self.integration_widget.overlay_tw.item(2, 2).text(), new_name)
        self.assertEqual(self.model.overlay_model.overlays[3].name, 'pattern_001')
        self.assertEqual(self.integration_widget.overlay_tw.item(3, 2).text(), 'pattern_001')

    def test_move_single_overlay_one_step_down(self):
        # setting up the test
        self.load_overlays()
        self.overlay_widget.select_overlay(3)
        new_name = 'special'
        self.assertEqual(self.integration_widget.overlay_tw.item(3, 2).text(), 'pattern_001')
        self.overlay_controller.rename_overlay(3, new_name)
        self.integration_widget.overlay_tw.item(3, 2).setText(new_name)
        self.assertEqual(self.model.overlay_model.overlays[3].name, new_name)
        self.assertEqual(self.integration_widget.overlay_tw.item(3, 2).text(), new_name)
        self.assertEqual(self.model.overlay_model.overlays[4].name, 'pattern_001')

        # moving the overlay
        self.integration_widget.overlay_move_down_btn.click()

        # test for wanted result
        self.assertEqual(self.model.overlay_model.overlays[4].name, new_name)
        self.assertEqual(self.integration_widget.overlay_tw.item(4, 2).text(), new_name)
        self.assertEqual(self.model.overlay_model.overlays[3].name, 'pattern_001')
        self.assertEqual(self.integration_widget.overlay_tw.item(3, 2).text(), 'pattern_001')

if __name__ == '__main__':
    unittest.main()



# -*- coding: utf8 -*-
# Dioptas - GUI program for fast processing of 2D X-ray data
# Copyright (C) 2017  Clemens Prescher (clemens.prescher@gmail.com)
# Institute for Geology and Mineralogy, University of Cologne
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import numpy as np
import time
import matplotlib.pyplot as plt
from pyFAI.azimuthalIntegrator import AzimuthalIntegrator

from Data.ImgCorrection import CbnCorrection

# defining geometry
image_shape = [2048, 2048] #pixel
detector_distance = 200 #mm
wavelength = 0.31 #angstrom
center_x = 1024 #pixel
center_y = 1024 #pixel
tilt = 0 #degree
rotation = 0 #degree
pixel_size = 79 #um

#some initialization
dummy_tth = np.linspace(0, 35, 2000)
dummy_int = np.ones(dummy_tth.shape)
geometry = AzimuthalIntegrator()
geometry.setFit2D(directDist=detector_distance,
                  centerX=center_x,
                  centerY=center_y,
                  tilt=tilt,
                  tiltPlanRotation=rotation,
                  pixelX=pixel_size,
                  pixelY=pixel_size)
geometry.wavelength = wavelength/1e10
dummy_img = geometry.calcfrom1d(dummy_tth, dummy_int, shape=image_shape, correctSolidAngle=True)


tth_array = geometry.twoThetaArray(image_shape)
azi_array = geometry.chiArray(image_shape)


cbn_correction = CbnCorrection(tth_array, azi_array,
                               diamond_thickness=2.2,
                               seat_thickness=5.3,
                               small_cbn_seat_radius=0.4,
                               large_cbn_seat_radius=1.95,
                               tilt=0,
                               tilt_rotation=0)

t1= time.time()
cbn_correction.get_data()
print("It took {0}s".format(time.time()-t1))




# -*- coding: utf8 -*-
# Dioptas - GUI program for fast processing of 2D X-ray data
# Copyright (C) 2017  Clemens Prescher (clemens.prescher@gmail.com)
# Institute for Geology and Mineralogy, University of Cologne
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
import gc
import shutil
import numpy as np
from mock import MagicMock

from ..utility import QtTest, click_button, click_checkbox

from qtpy import QtCore, QtWidgets
from qtpy.QtTest import QTest

from ...widgets.integration import IntegrationWidget
from ...controller.integration.ImageController import ImageController
from ...model.DioptasModel import DioptasModel

unittest_data_path = os.path.join(os.path.dirname(__file__), '../data')


class ImageControllerTest(QtTest):
    def setUp(self):
        self.working_dir = {'image': ''}

        self.widget = IntegrationWidget()
        self.model = DioptasModel()

        self.controller = ImageController(
            widget=self.widget,
            dioptas_model=self.model)

    def tearDown(self):
        if os.path.exists(os.path.join(unittest_data_path, 'image_003.tif')):
            os.remove(os.path.join(unittest_data_path, 'image_003.tif'))
        del self.widget
        del self.model
        del self.controller
        gc.collect()

    def test_automatic_file_processing(self):
        # get into a specific folder
        QtWidgets.QFileDialog.getOpenFileNames = MagicMock(
            return_value=[os.path.join(unittest_data_path, 'image_001.tif')])
        click_button(self.widget.load_img_btn)
        self.assertEqual(str(self.widget.img_filename_txt.text()), 'image_001.tif')
        self.assertEqual(self.model.working_directories['image'], unittest_data_path)

        # enable autoprocessing:
        QTest.mouseClick(self.widget.autoprocess_cb, QtCore.Qt.LeftButton,
                         pos=QtCore.QPoint(2, self.widget.autoprocess_cb.height() / 2.0))

        self.assertFalse(self.model.configurations[0].img_model._directory_watcher.signalsBlocked())
        self.assertFalse(
            self.model.configurations[0].img_model._directory_watcher._file_system_watcher.signalsBlocked())

        self.assertTrue(self.widget.autoprocess_cb.isChecked())
        self.assertTrue(self.model.img_model.autoprocess)

        shutil.copy2(os.path.join(unittest_data_path, 'image_001.tif'),
                     os.path.join(unittest_data_path, 'image_003.tif'))

        self.model.configurations[0].img_model._directory_watcher._file_system_watcher.directoryChanged.emit(
            unittest_data_path)

        self.assertEqual('image_003.tif', str(self.widget.img_filename_txt.text()))

    def test_configuration_selected_changes_mask_mode(self):
        self.model.add_configuration()
        click_button(self.widget.img_mask_btn)
        self.assertTrue(self.model.use_mask)

        self.model.select_configuration(0)
        self.assertFalse(self.model.use_mask)
        self.assertFalse(self.widget.img_mask_btn.isChecked())

    def test_configuration_selected_changes_mask_transparency(self):
        click_button(self.widget.img_mask_btn)
        self.model.add_configuration()
        click_button(self.widget.img_mask_btn)
        click_checkbox(self.widget.mask_transparent_cb)
        self.assertTrue(self.model.transparent_mask)

        self.model.select_configuration(0)
        self.assertFalse(self.model.transparent_mask)
        self.assertFalse(self.widget.mask_transparent_cb.isChecked())

    def test_configuration_selected_changed_autoprocessing_of_images(self):
        click_checkbox(self.widget.autoprocess_cb)
        self.model.add_configuration()

        self.assertFalse(self.model.img_model.autoprocess)
        self.assertFalse(self.widget.autoprocess_cb.isChecked())

        self.model.select_configuration(0)
        self.assertTrue(self.model.img_model.autoprocess)
        self.assertTrue(self.widget.autoprocess_cb.isChecked())

    def test_configuration_selected_changes_calibration_name(self):
        self.model.calibration_model.calibration_name = "calib1"
        self.model.add_configuration()
        self.model.calibration_model.calibration_name = "calib2"

        self.model.select_configuration(0)
        self.assertEqual(str(self.widget.calibration_lbl.text()), "calib1")

        self.model.select_configuration(1)
        self.assertEqual(str(self.widget.calibration_lbl.text()), "calib2")

    def test_configuration_selected_updates_mask_plot(self):
        self.model.mask_model.add_mask(os.path.join(unittest_data_path, 'test.mask'))
        click_button(self.widget.img_mask_btn)
        first_mask = self.model.mask_model.get_img()
        self.model.add_configuration()
        self.assertFalse(self.widget.img_mask_btn.isChecked())

        self.model.mask_model.mask_below_threshold(self.model.img_data, 1)
        second_mask = self.model.mask_model.get_img()
        click_button(self.widget.img_mask_btn)

        self.model.select_configuration(0)
        self.assertEqual(np.sum(self.widget.img_widget.mask_img_item.image-first_mask), 0)
        self.model.select_configuration(1)
        self.assertEqual(np.sum(self.widget.img_widget.mask_img_item.image-second_mask), 0)

    def test_configuration_selected_updates_mask_transparency(self):
        self.model.mask_model.add_mask(os.path.join(unittest_data_path, 'test.mask'))
        click_button(self.widget.img_mask_btn)
        self.model.add_configuration()

        self.assertFalse(self.widget.img_mask_btn.isChecked())
        self.assertFalse(self.widget.mask_transparent_cb.isVisible())

    def test_configuration_selected_updates_roi_mode(self):
        click_button(self.widget.img_roi_btn)
        self.assertTrue(self.widget.img_roi_btn.isChecked())
        self.assertTrue(self.widget.img_widget.roi in self.widget.img_widget.img_view_box.addedItems)

        self.model.add_configuration()
        self.assertFalse(self.widget.img_roi_btn.isChecked())
        self.assertFalse(self.widget.img_widget.roi in self.widget.img_widget.img_view_box.addedItems)

    def test_adding_images(self):
        QtWidgets.QFileDialog.getOpenFileNames = MagicMock(
            return_value=[os.path.join(unittest_data_path, 'image_001.tif')])
        click_button(self.widget.load_img_btn)
        data1 = np.copy(self.widget.img_widget.img_data).astype(np.uint32)
        click_checkbox(self.widget.img_batch_mode_add_rb)
        QtWidgets.QFileDialog.getOpenFileNames = MagicMock(
            return_value=[os.path.join(unittest_data_path, 'image_001.tif'),
                          os.path.join(unittest_data_path, 'image_001.tif')])
        click_button(self.widget.load_img_btn)
        self.assertTrue(np.array_equal(2 * data1, self.widget.img_widget.img_data))

    def test_load_image_with_manual_input_file_name(self):
        file_name = os.path.join(unittest_data_path, 'LaB6_40keV_MarCCD.tif')
        self.widget.img_filename_txt.setText(file_name)
        self.controller.filename_txt_changed()
        old_data = np.copy(self.model.img_data)
        file_name = os.path.join(unittest_data_path, 'CeO2_Pilatus1M.tif')
        self.widget.img_filename_txt.setText(file_name)
        self.controller.filename_txt_changed()
        new_data = self.model.img_data
        self.assertFalse(np.array_equal(old_data, new_data))
